#include "Instruction.hpp"

#include "InstructionSet.hpp"
#include "Keywords.hpp"
#include "Error.hpp"

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <ctype.h>

namespace SoftWire
{
	Instruction::Instruction(const Syntax &instruction) : syntax(instruction)
	{
		extractOperands(instruction.operands);

		syntaxMnemonic = false;
		syntaxSpecifier = false;
		syntaxFirstOperand = false;
		syntaxSecondOperand = false;
		syntaxThirdOperand = false;

		flags = instruction.flags;
		
		next = 0;
	}

	Instruction::~Instruction()
	{
		delete next;
	}

	void Instruction::extractOperands(const char *syntax)
	{
		assert(syntax);

		specifier = Specifier::UNKNOWN;
		firstOperand = Operand::VOID;
		secondOperand = Operand::VOID;
		thirdOperand = Operand::VOID;

		char *string = strdup(syntax);
		const char *token = strtok(string, " ,");

		if(!token)
		{
			return;
		}

		specifier = Specifier::scan(token);

		if(specifier != Specifier::UNKNOWN)
		{
			token = strtok(0, " ,");

			if(!token)
			{
				delete[] string;
				return;
			}
		}

		firstOperand = Operand::scanSyntax(token);

		if(firstOperand != Operand::UNKNOWN)
		{
			token = strtok(0, " ,");

			if(token == 0)
			{
				delete[] string;
				return;
			}
		}

		secondOperand = Operand::scanSyntax(token);

		if(secondOperand != Operand::UNKNOWN)
		{
			token = strtok(0, " ,");

			if(token == 0)
			{
				delete[] string;
				return;
			}
		}

		thirdOperand = Operand::scanSyntax(token);

		if(thirdOperand != Operand::UNKNOWN)
		{
			token = strtok(0, " ,");

			if(token == 0)
			{
				delete[] string;
				return;
			}
		}

		if(token == 0)
		{
			delete[] string;
			return;
		}
		else
		{
			throw Error("Invalid operand encoding '%s %s'", syntax);
		}
	}

	void Instruction::attachNew(const Syntax &instruction)
	{
		if(!next)
		{
			next = new Instruction(instruction);
		}
		else
		{
			next->attachNew(instruction);
		}
	}

	Instruction *Instruction::getNext() const
	{
		return next;
	}

	bool Instruction::matchSyntax() const
	{
		return syntaxMnemonic == true &&
		       syntaxSpecifier == true &&
		       syntaxFirstOperand == true &&
		       syntaxSecondOperand == true &&
		       syntaxThirdOperand == true;	
	}

	void Instruction::resetMatch()
	{
		syntaxMnemonic = false;
		syntaxSpecifier = false;
		syntaxFirstOperand = false;
		syntaxSecondOperand = false;
		syntaxThirdOperand = false;

		if(next)
		{
			next->resetMatch();
		}
	}

	void Instruction::matchMnemonic(const char *mnemonic)
	{
		if(stricmp(syntax.mnemonic, mnemonic) == 0)
		{
			syntaxMnemonic = true;
		}

		if(next)
		{
			next->matchMnemonic(mnemonic);
		}
	}

	void Instruction::matchSpecifier(Specifier::Type specifier)
	{
		if(this->specifier == Specifier::UNKNOWN)
		{
			if(specifier != Specifier::UNKNOWN)
			{
				if(firstOperand == Operand::R_M8 || secondOperand == Operand::R_M8)
				{
					syntaxSpecifier = specifier == Specifier::BYTE;
				}
				else if(firstOperand == Operand::R_M16 || secondOperand == Operand::R_M16)
				{
					syntaxSpecifier = specifier == Specifier::WORD;
				}
				else if(firstOperand == Operand::R_M32 || secondOperand == Operand::R_M32)
				{
					syntaxSpecifier = specifier == Specifier::DWORD;
				}
				else if(firstOperand == Operand::R_M64 || secondOperand == Operand::R_M64)
				{
					syntaxSpecifier = (specifier == Specifier::QWORD || specifier == Specifier::MMWORD);
				}
				else if(firstOperand == Operand::R_M128 || secondOperand == Operand::R_M128)
				{
					syntaxSpecifier = specifier == Specifier::XMMWORD;
				}
				else
				{
					syntaxSpecifier = true;
				}
			}
			else
			{
				syntaxSpecifier = true;
			}
		}
		else if(this->specifier != Specifier::UNKNOWN)   // Explicit specifier
		{
			if(this->specifier == specifier)
			{
				syntaxSpecifier = true;
			}
			else if(specifier == Specifier::UNKNOWN)   // Specifiers are optional
			{
				syntaxSpecifier = true;
			}
			else
			{
				syntaxSpecifier = false;
			}
		}

		if(next)
		{
			next->matchSpecifier(specifier);
		}
	}

	void Instruction::matchFirstOperand(const Operand &operand)
	{
		if(operand.isSubtypeOf(firstOperand))
		{
			syntaxFirstOperand = true;
		}
		else if(operand.type == Operand::MEM && firstOperand & Operand::MEM)
		{
			if(syntaxSpecifier)   // Explicit size specfier
			{
				syntaxFirstOperand = true;
			}
			else if(secondOperand != Operand::UNKNOWN)   // Implicit size specifier
			{
				syntaxFirstOperand = true;
			}
		}

		if(next)
		{
			next->matchFirstOperand(operand);
		}
	}

	void Instruction::matchSecondOperand(const Operand &operand)
	{
		if(operand.isSubtypeOf(secondOperand))
		{
			syntaxSecondOperand = true;
		}
		else if(operand.type == Operand::MEM && secondOperand & Operand::MEM)
		{
			if(syntaxSpecifier)   // Explicit size specfier
			{
				syntaxSecondOperand = true;
			}
			else if(firstOperand != Operand::UNKNOWN)   // Implicit size specifier
			{
				syntaxSecondOperand = true;
			}
		}

		if(next)
		{
			next->matchSecondOperand(operand);
		}
	}

	void Instruction::matchThirdOperand(const Operand &operand)
	{
		if(operand.isSubtypeOf(thirdOperand))
		{
			syntaxThirdOperand = true;
		}

		if(next)
		{
			next->matchThirdOperand(operand);
		}
	}

	const char *Instruction::getMnemonic() const
	{
		return syntax.mnemonic;
	}

	Operand::Type Instruction::getFirstOperand() const
	{
		return firstOperand;
	}

	Operand::Type Instruction::getSecondOperand() const
	{
		return secondOperand;
	}

	Operand::Type Instruction::getThirdOperand() const
	{
		return thirdOperand;
	}

	const char *Instruction::getOperandSyntax() const
	{
		return syntax.operands;
	}

	const char *Instruction::getEncoding() const
	{
		return syntax.encoding;
	}

	bool Instruction::is32Bit() const
	{
		return (flags & CPU_386) == CPU_386;
	}
}